@@ -22,17 +22,17 @@
 
 namespace MediaWiki\Revision;
 
-use CommentStoreComment;
 use Content;
 use InvalidArgumentException;
+use MediaWiki\CommentStore\CommentStoreComment;
 use MediaWiki\DAO\WikiAwareEntity;
 use MediaWiki\DAO\WikiAwareEntityTrait;
 use MediaWiki\Linker\LinkTarget;
 use MediaWiki\Page\LegacyArticleIdAccess;
 use MediaWiki\Page\PageIdentity;
 use MediaWiki\Permissions\Authority;
+use MediaWiki\Title\Title;
 use MediaWiki\User\UserIdentity;
-use Title;
 use Wikimedia\NonSerializable\NonSerializableTrait;
 
 /**
@@ -145,21 +145,40 @@ abstract class RevisionRecord implements WikiAwareEntity {
 	 * Note that for mutable Content objects, each call to this method will return a
 	 * fresh clone.
 	 *
-	 * MCR migration note: this replaced Revision::getContent
+	 * Use getContentOrThrow() for more specific error information.
 	 *
 	 * @param string $role The role name of the desired slot
 	 * @param int $audience
 	 * @param Authority|null $performer user on whose behalf to check
 	 *
-	 * @return Content|null The content of the given slot, or null if access is forbidden.
+	 * @return Content|null The content of the given slot, or null on error
 	 */
 	public function getContent( $role, $audience = self::FOR_PUBLIC, Authority $performer = null ): ?Content {
-		// XXX: throwing an exception would be nicer, but would a further
-		// departure from the old signature of Revision::getContent() when it existed,
-		// and thus result in more complex and error prone refactoring.
-		if ( !$this->audienceCan( self::DELETED_TEXT, $audience, $performer ) ) {
+		try {
+			$content = $this->getSlot( $role, $audience, $performer )->getContent();
+		} catch ( BadRevisionException | SuppressedDataException $e ) {
 			return null;
 		}
+		return $content->copy();
+	}
+
+	/**
+	 * Get the Content of the given slot of this revision.
+	 *
+	 * @param string $role The role name of the desired slot
+	 * @param int $audience
+	 * @param Authority|null $performer user on whose behalf to check
+	 *
+	 * @return Content
+	 * @throws SuppressedDataException if the content is not viewable by the given audience
+	 * @throws BadRevisionException if the content is missing or corrupted
+	 * @throws RevisionAccessException
+	 */
+	public function getContentOrThrow( $role, $audience = self::FOR_PUBLIC, Authority $performer = null ): Content {
+		if ( !$this->audienceCan( self::DELETED_TEXT, $audience, $performer ) ) {
+			throw new SuppressedDataException(
+				'Access to the content has been suppressed for this audience' );
+		}
 
 		$content = $this->getSlot( $role, $audience, $performer )->getContent();
 		return $content->copy();
@@ -170,7 +189,7 @@ abstract class RevisionRecord implements WikiAwareEntity {
 	 *
 	 * @param string $role The role name of the desired slot
 	 * @param int $audience
-	 * @param Authority|null $performer user on who's behalf to check
+	 * @param Authority|null $performer user on whose behalf to check
 	 *
 	 * @throws RevisionAccessException if the slot does not exist or slot data
 	 *        could not be lazy-loaded.
@@ -355,8 +374,7 @@ abstract class RevisionRecord implements WikiAwareEntity {
 	public function getPageAsLinkTarget() {
 		// TODO: Should be TitleValue::newFromPage( $this->mPage ),
 		// but Title is used too much still, so let's keep propagating it
-		// @phan-suppress-next-line PhanTypeMismatchReturnNullable castFrom does not return null here
-		return Title::castFromPageIdentity( $this->mPage );
+		return Title::newFromPageIdentity( $this->mPage );
 	}
 
 	/**
@@ -384,7 +402,7 @@ abstract class RevisionRecord implements WikiAwareEntity {
 	 *   RevisionRecord::FOR_PUBLIC       to be displayed to all users
 	 *   RevisionRecord::FOR_THIS_USER    to be displayed to the given user
 	 *   RevisionRecord::RAW              get the ID regardless of permissions
-	 * @param Authority|null $performer user on who's behalf to check
+	 * @param Authority|null $performer user on whose behalf to check
 	 * @return UserIdentity|null
 	 */
 	public function getUser( $audience = self::FOR_PUBLIC, Authority $performer = null ) {
@@ -407,7 +425,7 @@ abstract class RevisionRecord implements WikiAwareEntity {
 	 *   RevisionRecord::FOR_PUBLIC       to be displayed to all users
 	 *   RevisionRecord::FOR_THIS_USER    to be displayed to the given user
 	 *   RevisionRecord::RAW              get the text regardless of permissions
-	 * @param Authority|null $performer user on who's behalf to check
+	 * @param Authority|null $performer user on whose behalf to check
 	 *
 	 * @return CommentStoreComment|null
 	 */
@@ -473,7 +491,7 @@ abstract class RevisionRecord implements WikiAwareEntity {
 	 *        RevisionRecord::FOR_PUBLIC       to be displayed to all users
 	 *        RevisionRecord::FOR_THIS_USER    to be displayed to the given user
 	 *        RevisionRecord::RAW              get the text regardless of permissions
-	 * @param Authority|null $performer user on who's behalf to check
+	 * @param Authority|null $performer user on whose behalf to check
 	 *
 	 * @return bool
 	 */
@@ -504,7 +522,7 @@ abstract class RevisionRecord implements WikiAwareEntity {
 	 * @param int $field One of self::DELETED_TEXT,
 	 *                              self::DELETED_COMMENT,
 	 *                              self::DELETED_USER
-	 * @param Authority $performer user on who's behalf to check
+	 * @param Authority $performer user on whose behalf to check
 	 * @return bool
 	 */
 	public function userCan( $field, Authority $performer ) {
@@ -522,7 +540,7 @@ abstract class RevisionRecord implements WikiAwareEntity {
 	 * @param int $field One of self::DELETED_TEXT = File::DELETED_FILE,
 	 *                               self::DELETED_COMMENT = File::DELETED_COMMENT,
 	 *                               self::DELETED_USER = File::DELETED_USER
-	 * @param Authority $performer user on who's behalf to check
+	 * @param Authority $performer user on whose behalf to check
 	 * @param PageIdentity|null $page A PageIdentity object to check for per-page restrictions on,
 	 *                          instead of just plain user rights
 	 * @return bool
