@@ -21,6 +21,9 @@
  */
 
 use MediaWiki\ChangeTags\Taggable;
+use MediaWiki\Config\Config;
+use MediaWiki\Deferred\DeferredUpdates;
+use MediaWiki\HookContainer\HookRunner;
 use MediaWiki\MainConfigNames;
 use MediaWiki\MediaWikiServices;
 use MediaWiki\Page\PageIdentity;
@@ -29,8 +32,10 @@ use MediaWiki\Page\PageReferenceValue;
 use MediaWiki\Permissions\Authority;
 use MediaWiki\Permissions\PermissionStatus;
 use MediaWiki\Storage\EditResult;
+use MediaWiki\Title\Title;
 use MediaWiki\User\UserIdentity;
 use MediaWiki\User\UserIdentityValue;
+use MediaWiki\Utils\MWTimestamp;
 use Wikimedia\Assert\Assert;
 use Wikimedia\AtEase\AtEase;
 use Wikimedia\IPUtils;
@@ -166,8 +171,7 @@ class RecentChange implements Taggable {
 	/**
 	 * Parsing text to RC_* constants
 	 * @since 1.24
-	 * @param string|array $type
-	 * @throws MWException
+	 * @param string|array $type Callers must make sure that the given types are valid RC types.
 	 * @return int|array RC_TYPE
 	 */
 	public static function parseToRCType( $type ) {
@@ -181,7 +185,7 @@ class RecentChange implements Taggable {
 		}
 
 		if ( !array_key_exists( $type, self::CHANGE_TYPES ) ) {
-			throw new MWException( "Unknown type '$type'" );
+			throw new InvalidArgumentException( "Unknown type '$type'" );
 		}
 		return self::CHANGE_TYPES[$type];
 	}
@@ -231,7 +235,10 @@ class RecentChange implements Taggable {
 		$fname = __METHOD__,
 		$dbType = DB_REPLICA
 	) {
-		$db = wfGetDB( $dbType );
+		$icp = MediaWikiServices::getInstance()->getConnectionProvider();
+
+		$db = ( $dbType === DB_REPLICA ) ? $icp->getReplicaDatabase() : $icp->getPrimaryDatabase();
+
 		$rcQuery = self::getQueryInfo();
 		$row = $db->selectRow(
 			$rcQuery['tables'], $rcQuery['fields'], $conds, $fname, [], $rcQuery['joins']
@@ -259,7 +266,7 @@ class RecentChange implements Taggable {
 	 * @phan-return array{tables:string[],fields:string[],joins:array}
 	 */
 	public static function getQueryInfo() {
-		$commentQuery = CommentStore::getStore()->getJoin( 'rc_comment' );
+		$commentQuery = MediaWikiServices::getInstance()->getCommentStore()->getJoin( 'rc_comment' );
 		// Optimizer sometimes refuses to pick up the correct join order (T311360)
 		$commentQuery['joins']['comment_rc_comment'][0] = 'STRAIGHT_JOIN';
 		return [
@@ -365,21 +372,6 @@ class RecentChange implements Taggable {
 		return $this->mPage;
 	}
 
-	/**
-	 * Get the User object of the person who performed this change.
-	 * @deprecated since 1.36, hard deprecated since 1.37, use getPerformerIdentity() instead.
-	 *
-	 * @return User
-	 */
-	public function getPerformer(): User {
-		wfDeprecated( __METHOD__, '1.36' );
-		if ( !$this->mPerformer instanceof User ) {
-			$this->mPerformer = User::newFromIdentity( $this->getPerformerIdentity() );
-		}
-
-		return $this->mPerformer;
-	}
-
 	/**
 	 * Get the UserIdentity of the client that performed this change.
 	 *
@@ -409,9 +401,10 @@ class RecentChange implements Taggable {
 	 * @param bool $send self::SEND_FEED or self::SEND_NONE
 	 */
 	public function save( $send = self::SEND_FEED ) {
-		$mainConfig = MediaWikiServices::getInstance()->getMainConfig();
+		$services = MediaWikiServices::getInstance();
+		$mainConfig = $services->getMainConfig();
 		$putIPinRC = $mainConfig->get( MainConfigNames::PutIPinRC );
-		$dbw = wfGetDB( DB_PRIMARY );
+		$dbw = $services->getConnectionProvider()->getPrimaryDatabase();
 		if ( !is_array( $this->mExtra ) ) {
 			$this->mExtra = [];
 		}
@@ -434,7 +427,7 @@ class RecentChange implements Taggable {
 		$row = $this->mAttribs;
 
 		# Trim spaces on user supplied text
-		$row['rc_comment'] = trim( $row['rc_comment'] );
+		$row['rc_comment'] = trim( $row['rc_comment'] ?? '' );
 
 		# Fixup database timestamps
 		$row['rc_timestamp'] = $dbw->timestamp( $row['rc_timestamp'] );
@@ -447,12 +440,11 @@ class RecentChange implements Taggable {
 		# Convert mAttribs['rc_comment'] for CommentStore
 		$comment = $row['rc_comment'];
 		unset( $row['rc_comment'], $row['rc_comment_text'], $row['rc_comment_data'] );
-		$row += CommentStore::getStore()->insert( $dbw, 'rc_comment', $comment );
+		$row += $services->getCommentStore()->insert( $dbw, 'rc_comment', $comment );
 
 		# Normalize UserIdentity to actor ID
 		$user = $this->getPerformerIdentity();
-		$actorStore = MediaWikiServices::getInstance()->getActorStore();
-		$row['rc_actor'] = $actorStore->acquireActorId( $user, $dbw );
+		$row['rc_actor'] = $services->getActorStore()->acquireActorId( $user, $dbw );
 		unset( $row['rc_user'], $row['rc_user_text'] );
 
 		# Don't reuse an existing rc_id for the new row, if one happens to be
@@ -460,13 +452,17 @@ class RecentChange implements Taggable {
 		unset( $row['rc_id'] );
 
 		# Insert new row
-		$dbw->insert( 'recentchanges', $row, __METHOD__ );
+		$dbw->newInsertQueryBuilder()
+			->insertInto( 'recentchanges' )
+			->row( $row )
+			->caller( __METHOD__ )->execute();
 
 		# Set the ID
 		$this->mAttribs['rc_id'] = $dbw->insertId();
 
 		# Notify extensions
-		Hooks::runner()->onRecentChange_save( $this );
+		$hookRunner = new HookRunner( $services->getHookContainer() );
+		$hookRunner->onRecentChange_save( $this );
 
 		// Apply revert tags (if needed)
 		if ( $this->editResult !== null && count( $this->editResult->getRevertTags() ) ) {
@@ -499,11 +495,8 @@ class RecentChange implements Taggable {
 		}
 
 		# E-mail notifications
-		if ( $mainConfig->get( MainConfigNames::EnotifUserTalk ) ||
-			$mainConfig->get( MainConfigNames::EnotifWatchlist ) ||
-			$mainConfig->get( MainConfigNames::ShowUpdatedMarker )
-		) {
-			$userFactory = MediaWikiServices::getInstance()->getUserFactory();
+		if ( self::isEnotifEnabled( $mainConfig ) ) {
+			$userFactory = $services->getUserFactory();
 			$editor = $userFactory->newFromUserIdentity( $this->getPerformerIdentity() );
 			$page = $this->getPage();
 			$title = Title::castFromPageReference( $page );
@@ -511,7 +504,7 @@ class RecentChange implements Taggable {
 			// Never send an RC notification email about categorization changes
 			if (
 				$title &&
-				Hooks::runner()->onAbortEmailNotification( $editor, $title, $this ) &&
+				$hookRunner->onAbortEmailNotification( $editor, $title, $this ) &&
 				$this->mAttribs['rc_type'] != RC_CATEGORIZE
 			) {
 				// @FIXME: This would be better as an extension hook
@@ -543,7 +536,7 @@ class RecentChange implements Taggable {
 		if ( $this->mAttribs['rc_user'] > 0 ) {
 			$jobs[] = RecentChangesUpdateJob::newCacheUpdateJob();
 		}
-		MediaWikiServices::getInstance()->getJobQueueGroup()->lazyPush( $jobs );
+		$services->getJobQueueGroup()->lazyPush( $jobs );
 	}
 
 	/**
@@ -551,17 +544,8 @@ class RecentChange implements Taggable {
 	 * @param array|null $feeds Optional feeds to send to, defaults to $wgRCFeeds
 	 */
 	public function notifyRCFeeds( array $feeds = null ) {
-		// T403757: Don't send 'suppressed from creation' recent changes entries to the RCFeeds as they do not
-		// have systems to appropriately redact suppressed / deleted material
-		if ( $this->mAttribs['rc_deleted'] != 0 ) {
-			return;
-		}
-
-		$rcFeeds =
+		$feeds ??=
 			MediaWikiServices::getInstance()->getMainConfig()->get( MainConfigNames::RCFeeds );
-		if ( $feeds === null ) {
-			$feeds = $rcFeeds;
-		}
 
 		$performer = $this->getPerformerIdentity();
 
@@ -592,45 +576,24 @@ class RecentChange implements Taggable {
 		}
 	}
 
-	/**
-	 * @since 1.22
-	 * @codeCoverageIgnore
-	 * @deprecated since 1.29 Use RCFeed::factory() instead. Hard deprecated since 1.38.
-	 * @param string $uri URI to get the engine object for
-	 * @param array $params
-	 * @return FormattedRCFeed
-	 * @throws MWException
-	 */
-	public static function getEngine( $uri, $params = [] ) {
-		wfDeprecated( __METHOD__, '1.29' );
-		$rcEngines =
-			MediaWikiServices::getInstance()->getMainConfig()->get( MainConfigNames::RCEngines );
-		$scheme = parse_url( $uri, PHP_URL_SCHEME );
-		if ( !$scheme ) {
-			throw new MWException( "Invalid RCFeed uri: '$uri'" );
-		}
-		if ( !isset( $rcEngines[$scheme] ) ) {
-			throw new MWException( "Unknown RCFeed engine: '$scheme'" );
-		}
-		if ( defined( 'MW_PHPUNIT_TEST' ) && is_object( $rcEngines[$scheme] ) ) {
-			return $rcEngines[$scheme];
-		}
-		return new $rcEngines[$scheme]( $params );
-	}
-
 	/**
 	 * Mark this RecentChange as patrolled
 	 *
 	 * NOTE: Can also return 'rcpatroldisabled', 'hookaborted' and
 	 * 'markedaspatrollederror-noautopatrol' as errors
 	 * @param Authority $performer User performing the action
-	 * @param bool $auto For automatic patrol
+	 * @param bool|null $auto Unused. Passing true logs a warning.
 	 * @param string|string[]|null $tags Change tags to add to the patrol log entry
 	 *   ($user should be able to add the specified tags before this is called)
 	 * @return array[] Array of permissions errors, see PermissionManager::getPermissionErrors()
 	 */
-	public function doMarkPatrolled( Authority $performer, $auto = false, $tags = null ) {
-		$mainConfig = MediaWikiServices::getInstance()->getMainConfig();
+	public function doMarkPatrolled( Authority $performer, $auto = null, $tags = null ) {
+		if ( $auto ) {
+			wfWarn( __METHOD__ . ' with $auto = true' );
+			return [];
+		}
+		$services = MediaWikiServices::getInstance();
+		$mainConfig = $services->getMainConfig();
 		$useRCPatrol = $mainConfig->get( MainConfigNames::UseRCPatrol );
 		$useNPPatrol = $mainConfig->get( MainConfigNames::UseNPPatrol );
 		$useFilePatrol = $mainConfig->get( MainConfigNames::UseFilePatrol );
@@ -649,11 +612,11 @@ class RecentChange implements Taggable {
 			$this->getAttribute( 'rc_log_type' ) == 'upload' ) ) ) {
 			$status->fatal( 'rcpatroldisabled' );
 		}
-		// Automatic patrol needs "autopatrol", ordinary patrol needs "patrol"
-		$performer->authorizeWrite( $auto ? 'autopatrol' : 'patrol', $this->getTitle(), $status );
-		$user = MediaWikiServices::getInstance()->getUserFactory()->newFromAuthority( $performer );
-		if ( !Hooks::runner()->onMarkPatrolled(
-			$this->getAttribute( 'rc_id' ), $user, false, $auto, $tags )
+		$performer->authorizeWrite( 'patrol', $this->getTitle(), $status );
+		$user = $services->getUserFactory()->newFromAuthority( $performer );
+		$hookRunner = new HookRunner( $services->getHookContainer() );
+		if ( !$hookRunner->onMarkPatrolled(
+			$this->getAttribute( 'rc_id' ), $user, false, false, $tags )
 		) {
 			$status->fatal( 'hookaborted' );
 		}
@@ -670,33 +633,46 @@ class RecentChange implements Taggable {
 		if ( $this->getAttribute( 'rc_patrolled' ) ) {
 			return [];
 		}
-		// Actually set the 'patrolled' flag in RC
-		$this->reallyMarkPatrolled();
-		// Log this patrol event
-		PatrolLog::record( $this, $auto, $performer->getUser(), $tags );
+		// Attempt to set the 'patrolled' flag in RC database
+		$affectedRowCount = $this->reallyMarkPatrolled();
 
-		Hooks::runner()->onMarkPatrolledComplete(
-			$this->getAttribute( 'rc_id' ), $user, false, $auto );
+		if ( $affectedRowCount === 0 ) {
+			// Query succeeded but no rows change, e.g. another request
+			// patrolled the same change just before us.
+			// Avoid duplicate log entry (T196182).
+			return [];
+		}
+
+		// Log this patrol event
+		PatrolLog::record( $this, false, $performer->getUser(), $tags );
+
+		$hookRunner->onMarkPatrolledComplete(
+			$this->getAttribute( 'rc_id' ), $user, false, false );
 
 		return [];
 	}
 
 	/**
 	 * Mark this RecentChange patrolled, without error checking
-	 * @return int Number of affected rows
+	 *
+	 * @return int Number of database rows changed, usually 1, but 0 if
+	 * another request already patrolled it in the mean time.
 	 */
 	public function reallyMarkPatrolled() {
-		$dbw = wfGetDB( DB_PRIMARY );
-		$dbw->update(
-			'recentchanges',
-			[
-				'rc_patrolled' => self::PRC_PATROLLED
-			],
-			[
-				'rc_id' => $this->getAttribute( 'rc_id' )
-			],
-			__METHOD__
-		);
+		$dbw = MediaWikiServices::getInstance()->getConnectionProvider()->getPrimaryDatabase();
+		$dbw->newUpdateQueryBuilder()
+			->update( 'recentchanges' )
+			->set( [ 'rc_patrolled' => self::PRC_PATROLLED ] )
+			->where( [
+				'rc_id' => $this->getAttribute( 'rc_id' ),
+				'rc_patrolled' => self::PRC_UNPATROLLED,
+			] )
+			->caller( __METHOD__ )->execute();
+		$affectedRowCount = $dbw->affectedRows();
+		// The change was patrolled already, do nothing
+		if ( $affectedRowCount === 0 ) {
+			return 0;
+		}
 		// Invalidate the page cache after the page has been patrolled
 		// to make sure that the Patrol link isn't visible any longer!
 		$this->getTitle()->invalidateCache();
@@ -709,7 +685,7 @@ class RecentChange implements Taggable {
 			$revertedTagUpdateManager->approveRevertedTagForRevision( $revisionId );
 		}
 
-		return $dbw->affectedRows();
+		return $affectedRowCount;
 	}
 
 	/**
@@ -792,7 +768,7 @@ class RecentChange implements Taggable {
 				$rc->save();
 			},
 			DeferredUpdates::POSTSEND,
-			wfGetDB( DB_PRIMARY )
+			MediaWikiServices::getInstance()->getConnectionProvider()->getPrimaryDatabase()
 		);
 
 		return $rc;
@@ -872,7 +848,7 @@ class RecentChange implements Taggable {
 				$rc->save();
 			},
 			DeferredUpdates::POSTSEND,
-			wfGetDB( DB_PRIMARY )
+			MediaWikiServices::getInstance()->getConnectionProvider()->getPrimaryDatabase()
 		);
 
 		return $rc;
@@ -930,16 +906,16 @@ class RecentChange implements Taggable {
 	 * @param bool $isPatrollable Whether this log entry is patrollable
 	 * @param bool|null $forceBotFlag Override the default behavior and set bot flag to
 	 * 	the value of the argument. When omitted or null, it falls back to the global state.
-	 * @param int|null $deleted Value to set as rc_deleted (one of the LogPage::DELETED_* constants or 0)
 	 *
 	 * @return RecentChange
 	 */
 	public static function newLogEntry( $timestamp,
 		$logPage, $user, $actionComment, $ip,
 		$type, $action, $target, $logComment, $params, $newId = 0, $actionCommentIRC = '',
-		$revId = 0, $isPatrollable = false, $forceBotFlag = null, $deleted = 0
+		$revId = 0, $isPatrollable = false, $forceBotFlag = null
 	) {
 		global $wgRequest;
+
 		$permissionManager = MediaWikiServices::getInstance()->getPermissionManager();
 
 		# # Get pageStatus for email notification
@@ -1006,7 +982,7 @@ class RecentChange implements Taggable {
 			'rc_new' => 0, # obsolete
 			'rc_old_len' => null,
 			'rc_new_len' => null,
-			'rc_deleted' => $deleted,
+			'rc_deleted' => 0,
 			'rc_logid' => $newId,
 			'rc_log_type' => $type,
 			'rc_log_action' => $action,
@@ -1093,7 +1069,8 @@ class RecentChange implements Taggable {
 			'rc_source' => self::SRC_CATEGORIZE,
 			'rc_minor' => 0,
 			// XXX: rc_cur_id does not correspond to rc_namespace/rc_title.
-			//      They refer to different pages. Is that intentional?
+			// It's because when the page (rc_cur_id) is deleted, we want
+			// to delete the categorization entries, too (see LinksDeletionUpdate).
 			'rc_cur_id' => $pageTitle->getId(),
 			'rc_user' => $user->getId(),
 			'rc_user_text' => $user->getName(),
@@ -1153,9 +1130,10 @@ class RecentChange implements Taggable {
 		// rc_deleted MUST be set
 		$this->mAttribs['rc_deleted'] = $row->rc_deleted;
 
-		$comment = CommentStore::getStore()
+		$dbr = MediaWikiServices::getInstance()->getConnectionProvider()->getReplicaDatabase();
+		$comment = MediaWikiServices::getInstance()->getCommentStore()
 			// Legacy because $row may have come from self::selectFields()
-			->getCommentLegacy( wfGetDB( DB_REPLICA ), 'rc_comment', $row, true )
+			->getCommentLegacy( $dbr, 'rc_comment', $row, true )
 			->text;
 		$this->mAttribs['rc_comment'] = &$comment;
 		$this->mAttribs['rc_comment_text'] = &$comment;
@@ -1183,7 +1161,7 @@ class RecentChange implements Taggable {
 	 */
 	public function getAttribute( $name ) {
 		if ( $name === 'rc_comment' ) {
-			return CommentStore::getStore()
+			return MediaWikiServices::getInstance()->getCommentStore()
 				->getComment( 'rc_comment', $this->mAttribs, true )->text;
 		}
 
@@ -1200,7 +1178,7 @@ class RecentChange implements Taggable {
 				// NOTE: rc_actor exists in the database, but application logic should not use it.
 				wfDeprecatedMsg( 'Accessing deprecated field rc_actor', '1.36' );
 				$actorStore = MediaWikiServices::getInstance()->getActorStore();
-				$db = wfGetDB( DB_REPLICA );
+				$db = MediaWikiServices::getInstance()->getConnectionProvider()->getReplicaDatabase();
 				return $actorStore->findActorId( $user, $db );
 			}
 		}
@@ -1260,9 +1238,10 @@ class RecentChange implements Taggable {
 
 	private static function checkIPAddress( $ip ) {
 		global $wgRequest;
+
 		if ( $ip ) {
 			if ( !IPUtils::isIPAddress( $ip ) ) {
-				throw new MWException( "Attempt to write \"" . $ip .
+				throw new RuntimeException( "Attempt to write \"" . $ip .
 					"\" as an IP address into recent changes" );
 			}
 		} else {
@@ -1291,11 +1270,81 @@ class RecentChange implements Taggable {
 		return (int)wfTimestamp( TS_UNIX, $timestamp ) > time() - $tolerance - $rcMaxAge;
 	}
 
+	/**
+	 * Whether e-mail notifications are generally enabled on this wiki.
+	 *
+	 * This is used for:
+	 *
+	 * - performance optimization in RecentChange::save().
+	 *   After an edit, whether or not we need to use the EmailNotification
+	 *   service to determine which EnotifNotifyJob to dispatch.
+	 *
+	 * - performance optmization in WatchlistManager.
+	 *   After using reset ("Mark all pages as seen") on Special:Watchlist,
+	 *   whether to only look for user talk data to reset, or whether to look
+	 *   at all possible pages for timestamps to reset.
+	 *
+	 * TODO: Determine whether these optimizations still make sense.
+	 *
+	 * FIXME: The $wgShowUpdatedMarker variable was added to this condtion
+	 * in 2008 (2cf12c973d, SVN r35001) because at the time the per-user
+	 * "last seen" marker for watchlist and page history, was managed by
+	 * the EmailNotification/UserMailed classes. As of August 2022, this
+	 * appears to no longer be the case.
+	 *
+	 * @since 1.40
+	 * @param Config $conf
+	 * @return bool
+	 */
+	public static function isEnotifEnabled( Config $conf ): bool {
+		return $conf->get( MainConfigNames::EnotifUserTalk ) ||
+			$conf->get( MainConfigNames::EnotifWatchlist ) ||
+			$conf->get( MainConfigNames::ShowUpdatedMarker );
+	}
+
+	/**
+	 * Get the extra URL that is given as part of the notification to RCFeed consumers.
+	 *
+	 * This is mainly to facilitate patrolling or other content review.
+	 *
+	 * @since 1.40
+	 * @return string|null URL
+	 */
+	public function getNotifyUrl() {
+		$services = MediaWikiServices::getInstance();
+		$mainConfig = $services->getMainConfig();
+		$useRCPatrol = $mainConfig->get( MainConfigNames::UseRCPatrol );
+		$useNPPatrol = $mainConfig->get( MainConfigNames::UseNPPatrol );
+		$localInterwikis = $mainConfig->get( MainConfigNames::LocalInterwikis );
+		$canonicalServer = $mainConfig->get( MainConfigNames::CanonicalServer );
+		$script = $mainConfig->get( MainConfigNames::Script );
+
+		$type = $this->getAttribute( 'rc_type' );
+		if ( $type == RC_LOG ) {
+			$url = null;
+		} else {
+			$url = $canonicalServer . $script;
+			if ( $type == RC_NEW ) {
+				$query = '?oldid=' . $this->getAttribute( 'rc_this_oldid' );
+			} else {
+				$query = '?diff=' . $this->getAttribute( 'rc_this_oldid' )
+					. '&oldid=' . $this->getAttribute( 'rc_last_oldid' );
+			}
+			if ( $useRCPatrol || ( $this->getAttribute( 'rc_type' ) == RC_NEW && $useNPPatrol ) ) {
+				$query .= '&rcid=' . $this->getAttribute( 'rc_id' );
+			}
+
+			( new HookRunner( $services->getHookContainer() ) )->onIRCLineURL( $url, $query, $this );
+			$url .= $query;
+		}
+
+		return $url;
+	}
+
 	/**
 	 * Parses and returns the rc_params attribute
 	 *
 	 * @since 1.26
-	 *
 	 * @return mixed|bool false on failed unserialization
 	 */
 	public function parseParams() {
@@ -1370,7 +1419,7 @@ class RecentChange implements Taggable {
 				}
 			}
 		} elseif ( $actorId > 0 ) {
-			$db = wfGetDB( DB_REPLICA );
+			$db = MediaWikiServices::getInstance()->getConnectionProvider()->getReplicaDatabase();
 			$user = $actorStore->getActorById( $actorId, $db );
 
 			if ( !$user ) {
