@@ -16,7 +16,7 @@ use InvalidArgumentException;
  * @par Example:
  * @code
  *     function getRowData() {
- *         $conn = $this->lb->getConnectedRef( DB_REPLICA );
+ *         $conn = $this->lb->getConnectionRef( DB_REPLICA );
  *         $row = $conn->select( ... );
  *         return $row ? (array)$row : false;
  *         // $conn falls out of scope and $this->lb->reuseConnection() gets called
@@ -31,7 +31,10 @@ class DBConnRef implements IMaintainableDatabase {
 	private $lb;
 	/** @var Database|null Live connection handle */
 	private $conn;
-	/** @var array N-tuple of (server index, group, DatabaseDomain|string) */
+	/**
+	 * @var array Map of (DBConnRef::FLD_* constant => connection parameter)
+	 * @phan-var array{0:int,1:array|string|false,2:DatabaseDomain,3:int}
+	 */
 	private $params;
 	/** @var int One of DB_PRIMARY/DB_REPLICA */
 	private $role;
@@ -49,7 +52,7 @@ class DBConnRef implements IMaintainableDatabase {
 	private $modCountFix;
 
 	private const FLD_INDEX = 0;
-	private const FLD_GROUP = 1;
+	private const FLD_GROUPS = 1;
 	private const FLD_DOMAIN = 2;
 	private const FLD_FLAGS = 3;
 
@@ -66,6 +69,8 @@ class DBConnRef implements IMaintainableDatabase {
 			throw new InvalidArgumentException( "Missing lazy connection arguments." );
 		}
 
+		$params[self::FLD_DOMAIN] = DatabaseDomain::newFromId( $params[self::FLD_DOMAIN] );
+
 		$this->lb = $lb;
 		$this->params = $params;
 		$this->role = $role;
@@ -85,16 +90,27 @@ class DBConnRef implements IMaintainableDatabase {
 			// to take effect. The primary use case are replica servers being taken out of
 			// rotation, or the primary database changing.
 			if ( !$this->conn->trxLevel() ) {
-				$this->lb->closeConnection( $this->conn );
+				$this->conn->close();
 				$this->conn = null;
 			}
 		}
 
 		if ( $this->conn === null ) {
-			[ $index, $groups, $wiki, $flags ] = $this->params;
-			$this->conn = $this->lb->getConnectionInternal( $index, $groups, $wiki, $flags );
+			$this->conn = $this->lb->getConnectionInternal(
+				$this->params[self::FLD_INDEX],
+				$this->params[self::FLD_GROUPS],
+				$this->params[self::FLD_DOMAIN]->getId(),
+				$this->params[self::FLD_FLAGS]
+			);
 			$this->modCountFix = $this->modCountRef;
 		}
+
+		if ( !$this->params[self::FLD_DOMAIN]->equals( $this->conn->getDomainID() ) ) {
+			// The underlying connection handle is likely being shared by other DBConnRef
+			// instances in a load balancer. Make sure that each one routes queries by their
+			// owner function to the domain that the owner expects.
+			$this->conn->selectDomain( $this->params[self::FLD_DOMAIN] );
+		}
 	}
 
 	public function __call( $name, array $arguments ) {
@@ -143,8 +159,7 @@ class DBConnRef implements IMaintainableDatabase {
 
 		if ( $this->conn === null ) {
 			// Avoid triggering a database connection
-			$domain = DatabaseDomain::newFromId( $this->params[self::FLD_DOMAIN] );
-			$prefix = $domain->getTablePrefix();
+			$prefix = $this->params[self::FLD_DOMAIN]->getTablePrefix();
 		} else {
 			// This will just return the prefix
 			$prefix = $this->__call( __FUNCTION__, func_get_args() );
@@ -161,8 +176,7 @@ class DBConnRef implements IMaintainableDatabase {
 
 		if ( $this->conn === null ) {
 			// Avoid triggering a database connection
-			$domain = DatabaseDomain::newFromId( $this->params[self::FLD_DOMAIN] );
-			$schema = (string)$domain->getSchema();
+			$schema = (string)( $this->params[self::FLD_DOMAIN]->getSchema() );
 		} else {
 			// This will just return the schema
 			$schema = $this->__call( __FUNCTION__, func_get_args() );
@@ -185,10 +199,6 @@ class DBConnRef implements IMaintainableDatabase {
 		return $this->__call( __FUNCTION__, func_get_args() );
 	}
 
-	public function lastQuery() {
-		return $this->__call( __FUNCTION__, func_get_args() );
-	}
-
 	public function lastDoneWrites() {
 		return $this->__call( __FUNCTION__, func_get_args() );
 	}
@@ -235,9 +245,8 @@ class DBConnRef implements IMaintainableDatabase {
 
 	public function getDomainID() {
 		if ( $this->conn === null ) {
-			$domain = $this->params[self::FLD_DOMAIN];
 			// Avoid triggering a database connection
-			return $domain instanceof DatabaseDomain ? $domain->getId() : $domain;
+			return $this->params[self::FLD_DOMAIN]->getId();
 		}
 
 		return $this->__call( __FUNCTION__, func_get_args() );
@@ -293,20 +302,36 @@ class DBConnRef implements IMaintainableDatabase {
 		return $this->__call( __FUNCTION__, [ $sql, $fname, $flags ] );
 	}
 
-	public function queryMulti(
-		array $sqls, string $fname = __METHOD__, int $flags = 0, ?string $summarySql = null
-	) {
-		if ( $this->role !== ILoadBalancer::DB_PRIMARY ) {
-			$flags |= IDatabase::QUERY_REPLICA_ROLE;
-		}
-		return $this->__call( __FUNCTION__, [ $sqls, $fname, $flags, $summarySql ] );
-	}
-
 	public function newSelectQueryBuilder(): SelectQueryBuilder {
 		// Use $this not $this->conn so that the domain is preserved (T326377)
 		return new SelectQueryBuilder( $this );
 	}
 
+	public function newUnionQueryBuilder(): UnionQueryBuilder {
+		// Use $this not $this->conn so that the domain is preserved (T326377)
+		return new UnionQueryBuilder( $this );
+	}
+
+	public function newUpdateQueryBuilder(): UpdateQueryBuilder {
+		// Use $this not $this->conn so that the domain is preserved (T326377)
+		return new UpdateQueryBuilder( $this );
+	}
+
+	public function newDeleteQueryBuilder(): DeleteQueryBuilder {
+		// Use $this not $this->conn so that the domain is preserved (T326377)
+		return new DeleteQueryBuilder( $this );
+	}
+
+	public function newInsertQueryBuilder(): InsertQueryBuilder {
+		// Use $this not $this->conn so that the domain is preserved (T326377)
+		return new InsertQueryBuilder( $this );
+	}
+
+	public function newReplaceQueryBuilder(): ReplaceQueryBuilder {
+		// Use $this not $this->conn so that the domain is preserved (T326377)
+		return new ReplaceQueryBuilder( $this );
+	}
+
 	public function selectField(
 		$table, $var, $cond = '', $fname = __METHOD__, $options = [], $join_conds = []
 	) {
@@ -388,6 +413,10 @@ class DBConnRef implements IMaintainableDatabase {
 		return $this->__call( __FUNCTION__, func_get_args() );
 	}
 
+	public function buildComparison( string $op, array $conds ): string {
+		return $this->__call( __FUNCTION__, func_get_args() );
+	}
+
 	public function makeList( array $a, $mode = self::LIST_COMMA ) {
 		return $this->__call( __FUNCTION__, func_get_args() );
 	}
@@ -457,12 +486,6 @@ class DBConnRef implements IMaintainableDatabase {
 		return $this->__call( __FUNCTION__, func_get_args() );
 	}
 
-	public function selectDB( $db ) {
-		// @phan-suppress-previous-line PhanPluginNeverReturnMethod
-		// Disallow things that might confuse the LoadBalancer tracking
-		throw $this->getDomainChangeException();
-	}
-
 	public function selectDomain( $domain ) {
 		// @phan-suppress-previous-line PhanPluginNeverReturnMethod
 		// Disallow things that might confuse the LoadBalancer tracking
@@ -471,9 +494,8 @@ class DBConnRef implements IMaintainableDatabase {
 
 	public function getDBname() {
 		if ( $this->conn === null ) {
-			$domain = DatabaseDomain::newFromId( $this->params[self::FLD_DOMAIN] );
 			// Avoid triggering a database connection
-			return $domain->getDatabase();
+			return $this->params[self::FLD_DOMAIN]->getDatabase();
 		}
 
 		return $this->__call( __FUNCTION__, func_get_args() );
@@ -500,6 +522,10 @@ class DBConnRef implements IMaintainableDatabase {
 		return $this->__call( __FUNCTION__, func_get_args() );
 	}
 
+	public function expr( string $field, string $op, $value ): Expression {
+		return new Expression( $field, $op, $value );
+	}
+
 	public function addIdentifierQuotes( $s ) {
 		return $this->__call( __FUNCTION__, func_get_args() );
 	}
@@ -541,7 +567,7 @@ class DBConnRef implements IMaintainableDatabase {
 	) {
 		$this->assertRoleAllowsWrites();
 
-		return $this->__call( __FUNCTION__, func_get_args() );
+		$this->__call( __FUNCTION__, func_get_args() );
 	}
 
 	public function delete( $table, $conds, $fname = __METHOD__ ) {
@@ -563,14 +589,7 @@ class DBConnRef implements IMaintainableDatabase {
 		return $this->__call( __FUNCTION__, func_get_args() );
 	}
 
-	public function unionQueries( $sqls, $all ) {
-		return $this->__call( __FUNCTION__, func_get_args() );
-	}
-
-	public function unionConditionPermutations(
-		$table, $vars, array $permute_conds, $extra_conds = '', $fname = __METHOD__,
-		$options = [], $join_conds = []
-	) {
+	public function unionQueries( $sqls, $all, $options = [] ) {
 		return $this->__call( __FUNCTION__, func_get_args() );
 	}
 
@@ -586,22 +605,10 @@ class DBConnRef implements IMaintainableDatabase {
 		return $this->__call( __FUNCTION__, func_get_args() );
 	}
 
-	public function wasLockTimeout() {
-		return $this->__call( __FUNCTION__, func_get_args() );
-	}
-
-	public function wasConnectionLoss() {
-		return $this->__call( __FUNCTION__, func_get_args() );
-	}
-
 	public function wasReadOnlyError() {
 		return $this->__call( __FUNCTION__, func_get_args() );
 	}
 
-	public function wasErrorReissuable() {
-		return $this->__call( __FUNCTION__, func_get_args() );
-	}
-
 	public function primaryPosWait( DBPrimaryPos $pos, $timeout ) {
 		return $this->__call( __FUNCTION__, func_get_args() );
 	}
@@ -763,10 +770,6 @@ class DBConnRef implements IMaintainableDatabase {
 		return $this->__call( __FUNCTION__, func_get_args() );
 	}
 
-	public function setBigSelects( $value = true ) {
-		return $this->__call( __FUNCTION__, func_get_args() );
-	}
-
 	public function isReadOnly() {
 		return $this->__call( __FUNCTION__, func_get_args() );
 	}
@@ -825,13 +828,13 @@ class DBConnRef implements IMaintainableDatabase {
 		return $this->__call( __FUNCTION__, func_get_args() );
 	}
 
-	public function truncate( $tables, $fname = __METHOD__ ) {
+	public function truncateTable( $table, $fname = __METHOD__ ) {
 		$this->assertRoleAllowsWrites();
 
 		return $this->__call( __FUNCTION__, func_get_args() );
 	}
 
-	public function deadlockLoop( ...$args ) {
+	public function truncate( $tables, $fname = __METHOD__ ) {
 		$this->assertRoleAllowsWrites();
 
 		return $this->__call( __FUNCTION__, func_get_args() );
@@ -916,19 +919,4 @@ class DBConnRef implements IMaintainableDatabase {
 	protected function normalizeServerIndex( $i ) {
 		return ( $i === ILoadBalancer::DB_PRIMARY ) ? $this->lb->getWriterIndex() : $i;
 	}
-
-	/**
-	 * Clean up the connection when out of scope
-	 */
-	public function __destruct() {
-		if ( $this->conn ) {
-			$this->lb->reuseConnectionInternal( $this->conn );
-		}
-	}
 }
-
-/**
- * @since 1.22
- * @deprecated since 1.29
- */
-class_alias( DBConnRef::class, 'DBConnRef' );
