@@ -1,11 +1,27 @@
 <?php
 
-namespace MediaWiki\Auth;
+namespace MediaWiki\Tests\Auth;
 
+use BadMethodCallException;
+use MediaWiki\Auth\AuthenticationRequest;
+use MediaWiki\Auth\AuthenticationResponse;
+use MediaWiki\Auth\AuthManager;
+use MediaWiki\Auth\LocalPasswordPrimaryAuthenticationProvider;
+use MediaWiki\Auth\PasswordAuthenticationRequest;
+use MediaWiki\Auth\PasswordDomainAuthenticationRequest;
+use MediaWiki\Auth\PrimaryAuthenticationProvider;
+use MediaWiki\Config\HashConfig;
+use MediaWiki\Config\MultiConfig;
 use MediaWiki\MainConfigNames;
+use MediaWiki\Request\FauxRequest;
+use MediaWiki\Status\Status;
 use MediaWiki\Tests\Unit\Auth\AuthenticationProviderTestTrait;
+use MediaWiki\Tests\Unit\DummyServicesTrait;
+use MediaWiki\User\User;
 use MediaWiki\User\UserNameUtils;
-use Psr\Container\ContainerInterface;
+use MediaWikiIntegrationTestCase;
+use PasswordFactory;
+use StatusValue;
 use Wikimedia\TestingAccessWrapper;
 
 /**
@@ -13,8 +29,9 @@ use Wikimedia\TestingAccessWrapper;
  * @group Database
  * @covers \MediaWiki\Auth\LocalPasswordPrimaryAuthenticationProvider
  */
-class LocalPasswordPrimaryAuthenticationProviderTest extends \MediaWikiIntegrationTestCase {
+class LocalPasswordPrimaryAuthenticationProviderTest extends MediaWikiIntegrationTestCase {
 	use AuthenticationProviderTestTrait;
+	use DummyServicesTrait;
 
 	private $manager = null;
 	private $config = null;
@@ -32,9 +49,9 @@ class LocalPasswordPrimaryAuthenticationProviderTest extends \MediaWikiIntegrati
 	protected function getProvider( $loginOnly = false ) {
 		$mwServices = $this->getServiceContainer();
 		if ( !$this->config ) {
-			$this->config = new \HashConfig();
+			$this->config = new HashConfig();
 		}
-		$config = new \MultiConfig( [
+		$config = new MultiConfig( [
 			$this->config,
 			$mwServices->getMainConfig()
 		] );
@@ -44,14 +61,12 @@ class LocalPasswordPrimaryAuthenticationProviderTest extends \MediaWikiIntegrati
 		$hookContainer = $mwServices->getHookContainer();
 
 		if ( !$this->manager ) {
-			$services = $this->createNoOpAbstractMock( ContainerInterface::class );
-			$objectFactory = new \Wikimedia\ObjectFactory\ObjectFactory( $services );
 			$userNameUtils = $this->createNoOpMock( UserNameUtils::class );
 
 			$this->manager = new AuthManager(
-				new \FauxRequest(),
+				new FauxRequest(),
 				$config,
-				$objectFactory,
+				$this->getDummyObjectFactory(),
 				$hookContainer,
 				$mwServices->getReadOnlyMode(),
 				$userNameUtils,
@@ -66,11 +81,11 @@ class LocalPasswordPrimaryAuthenticationProviderTest extends \MediaWikiIntegrati
 				$mwServices->getUserOptionsManager()
 			);
 		}
-		$this->validity = \Status::newGood();
+		$this->validity = Status::newGood();
 		$provider = $this->getMockBuilder( LocalPasswordPrimaryAuthenticationProvider::class )
 			->onlyMethods( [ 'checkPasswordValidity' ] )
 			->setConstructorArgs( [
-				$mwServices->getDBLoadBalancer(),
+				$mwServices->getConnectionProvider(),
 				[ 'loginOnly' => $loginOnly ]
 			] )
 			->getMock();
@@ -122,7 +137,7 @@ class LocalPasswordPrimaryAuthenticationProviderTest extends \MediaWikiIntegrati
 	public function testTestUserCanAuthenticate() {
 		$user = $this->getMutableTestUser()->getUser();
 		$userName = $user->getName();
-		$dbw = wfGetDB( DB_PRIMARY );
+		$dbw = $this->getDb();
 
 		$provider = $this->getProvider();
 
@@ -136,7 +151,7 @@ class LocalPasswordPrimaryAuthenticationProviderTest extends \MediaWikiIntegrati
 
 		$dbw->update(
 			'user',
-			[ 'user_password' => \PasswordFactory::newInvalidPassword()->toString() ],
+			[ 'user_password' => PasswordFactory::newInvalidPassword()->toString() ],
 			[ 'user_name' => $userName ]
 		);
 		$this->assertFalse( $provider->testUserCanAuthenticate( $userName ) );
@@ -157,33 +172,31 @@ class LocalPasswordPrimaryAuthenticationProviderTest extends \MediaWikiIntegrati
 		// @todo: Because we're currently using User, which uses the global config...
 		$this->overrideConfigValue( MainConfigNames::PasswordExpireGrace, 100 );
 
-		$this->config->set( 'PasswordExpireGrace', 100 );
-		$this->config->set( 'InvalidPasswordReset', true );
+		$this->config->set( MainConfigNames::PasswordExpireGrace, 100 );
+		$this->config->set( MainConfigNames::InvalidPasswordReset, true );
 
 		$provider = new LocalPasswordPrimaryAuthenticationProvider(
-			$this->getServiceContainer()->getDBLoadBalancer()
+			$this->getServiceContainer()->getConnectionProvider()
 		);
 		$this->initProvider( $provider, $this->config, null, $this->manager );
 		$providerPriv = TestingAccessWrapper::newFromObject( $provider );
 
 		$user = $this->getMutableTestUser()->getUser();
 		$userName = $user->getName();
-		$dbw = wfGetDB( DB_PRIMARY );
-		$row = $dbw->selectRow(
-			'user',
-			'*',
-			[ 'user_name' => $userName ],
-			__METHOD__
-		);
+		$row = $this->getDb()->newSelectQueryBuilder()
+			->select( '*' )
+			->from( 'user' )
+			->where( [ 'user_name' => $userName ] )
+			->caller( __METHOD__ )->fetchRow();
 
 		$this->manager->removeAuthenticationSessionData( null );
 		$row->user_password_expires = wfTimestamp( TS_MW, time() + 200 );
-		$providerPriv->setPasswordResetFlag( $userName, \Status::newGood(), $row );
+		$providerPriv->setPasswordResetFlag( $userName, Status::newGood(), $row );
 		$this->assertNull( $this->manager->getAuthenticationSessionData( 'reset-pass' ) );
 
 		$this->manager->removeAuthenticationSessionData( null );
 		$row->user_password_expires = wfTimestamp( TS_MW, time() - 200 );
-		$providerPriv->setPasswordResetFlag( $userName, \Status::newGood(), $row );
+		$providerPriv->setPasswordResetFlag( $userName, Status::newGood(), $row );
 		$ret = $this->manager->getAuthenticationSessionData( 'reset-pass' );
 		$this->assertNotNull( $ret );
 		$this->assertSame( 'resetpass-expired', $ret->msg->getKey() );
@@ -191,7 +204,7 @@ class LocalPasswordPrimaryAuthenticationProviderTest extends \MediaWikiIntegrati
 
 		$this->manager->removeAuthenticationSessionData( null );
 		$row->user_password_expires = wfTimestamp( TS_MW, time() - 1 );
-		$providerPriv->setPasswordResetFlag( $userName, \Status::newGood(), $row );
+		$providerPriv->setPasswordResetFlag( $userName, Status::newGood(), $row );
 		$ret = $this->manager->getAuthenticationSessionData( 'reset-pass' );
 		$this->assertNotNull( $ret );
 		$this->assertSame( 'resetpass-expired-soft', $ret->msg->getKey() );
@@ -199,7 +212,7 @@ class LocalPasswordPrimaryAuthenticationProviderTest extends \MediaWikiIntegrati
 
 		$this->manager->removeAuthenticationSessionData( null );
 		$row->user_password_expires = null;
-		$status = \Status::newGood( [ 'suggestChangeOnLogin' => true ] );
+		$status = Status::newGood( [ 'suggestChangeOnLogin' => true ] );
 		$status->error( 'testing' );
 		$providerPriv->setPasswordResetFlag( $userName, $status, $row );
 		$ret = $this->manager->getAuthenticationSessionData( 'reset-pass' );
@@ -209,7 +222,7 @@ class LocalPasswordPrimaryAuthenticationProviderTest extends \MediaWikiIntegrati
 
 		$this->manager->removeAuthenticationSessionData( null );
 		$row->user_password_expires = null;
-		$status = \Status::newGood( [ 'forceChange' => true ] );
+		$status = Status::newGood( [ 'forceChange' => true ] );
 		$status->error( 'testing' );
 		$providerPriv->setPasswordResetFlag( $userName, $status, $row );
 		$ret = $this->manager->getAuthenticationSessionData( 'reset-pass' );
@@ -219,7 +232,7 @@ class LocalPasswordPrimaryAuthenticationProviderTest extends \MediaWikiIntegrati
 
 		$this->manager->removeAuthenticationSessionData( null );
 		$row->user_password_expires = null;
-		$status = \Status::newGood( [ 'suggestChangeOnLogin' => false, ] );
+		$status = Status::newGood( [ 'suggestChangeOnLogin' => false, ] );
 		$status->error( 'testing' );
 		$providerPriv->setPasswordResetFlag( $userName, $status, $row );
 		$ret = $this->manager->getAuthenticationSessionData( 'reset-pass' );
@@ -230,8 +243,8 @@ class LocalPasswordPrimaryAuthenticationProviderTest extends \MediaWikiIntegrati
 		$testUser = $this->getMutableTestUser();
 		$userName = $testUser->getUser()->getName();
 
-		$dbw = wfGetDB( DB_PRIMARY );
-		$id = \User::idFromName( $userName );
+		$dbw = $this->getDb();
+		$id = $testUser->getUser()->getId();
 
 		$req = new PasswordAuthenticationRequest();
 		$req->action = AuthManager::ACTION_LOGIN;
@@ -282,20 +295,22 @@ class LocalPasswordPrimaryAuthenticationProviderTest extends \MediaWikiIntegrati
 		// Validation failure
 		$req->username = $userName;
 		$req->password = $testUser->getPassword();
-		$this->validity = \Status::newFatal( 'arbitrary-failure' );
+		$this->validity = Status::newFatal( 'arbitrary-failure' );
 		$ret = $provider->beginPrimaryAuthentication( $reqs );
 		$this->assertEquals(
 			AuthenticationResponse::FAIL,
 			$ret->status
 		);
+		// AbstractPasswordPrimaryAuthenticationProvider::getFatalPasswordErrorResponse() will
+		// wrap the original message in 'fatalpassworderror'
 		$this->assertEquals(
-			'arbitrary-failure',
+			'fatalpassworderror',
 			$ret->message->getKey()
 		);
 
 		// Successful auth
 		$this->manager->removeAuthenticationSessionData( null );
-		$this->validity = \Status::newGood();
+		$this->validity = Status::newGood();
 		$this->assertEquals(
 			AuthenticationResponse::newPass( $userName ),
 			$provider->beginPrimaryAuthentication( $reqs )
@@ -304,7 +319,7 @@ class LocalPasswordPrimaryAuthenticationProviderTest extends \MediaWikiIntegrati
 
 		// Successful auth after normalizing name
 		$this->manager->removeAuthenticationSessionData( null );
-		$this->validity = \Status::newGood();
+		$this->validity = Status::newGood();
 		$req->username = mb_strtolower( $userName[0] ) . substr( $userName, 1 );
 		$this->assertEquals(
 			AuthenticationResponse::newPass( $userName ),
@@ -315,7 +330,7 @@ class LocalPasswordPrimaryAuthenticationProviderTest extends \MediaWikiIntegrati
 
 		// Successful auth with reset
 		$this->manager->removeAuthenticationSessionData( null );
-		$this->validity = \Status::newGood( [ 'suggestChangeOnLogin' => true ] );
+		$this->validity = Status::newGood( [ 'suggestChangeOnLogin' => true ] );
 		$this->validity->error( 'arbitrary-warning' );
 		$this->assertEquals(
 			AuthenticationResponse::newPass( $userName ),
@@ -324,7 +339,7 @@ class LocalPasswordPrimaryAuthenticationProviderTest extends \MediaWikiIntegrati
 		$this->assertNotNull( $this->manager->getAuthenticationSessionData( 'reset-pass' ) );
 
 		// Wrong password
-		$this->validity = \Status::newGood();
+		$this->validity = Status::newGood();
 		$req->password = 'Wrong';
 		$ret = $provider->beginPrimaryAuthentication( $reqs );
 		$this->assertEquals(
@@ -350,7 +365,7 @@ class LocalPasswordPrimaryAuthenticationProviderTest extends \MediaWikiIntegrati
 			$ret->message->getKey()
 		);
 
-		$this->config->set( 'LegacyEncoding', true );
+		$this->config->set( MainConfigNames::LegacyEncoding, true );
 		$this->assertEquals(
 			AuthenticationResponse::newPass( $userName ),
 			$provider->beginPrimaryAuthentication( $reqs )
@@ -368,7 +383,6 @@ class LocalPasswordPrimaryAuthenticationProviderTest extends \MediaWikiIntegrati
 		);
 
 		// Correct handling of really old password hashes
-		$this->config->set( 'PasswordSalt', true );
 		$password = md5( "$id-" . md5( 'FooBar' ) );
 		$dbw->update( 'user', [ 'user_password' => $password ], [ 'user_name' => $userName ] );
 		$req->password = 'FooBar';
@@ -380,15 +394,19 @@ class LocalPasswordPrimaryAuthenticationProviderTest extends \MediaWikiIntegrati
 
 	/**
 	 * @dataProvider provideProviderAllowsAuthenticationDataChange
+	 *
 	 * @param string $type
-	 * @param string $user
-	 * @param \Status $validity Result of the password validity check
-	 * @param \StatusValue $expect1 Expected result with $checkData = false
-	 * @param \StatusValue $expect2 Expected result with $checkData = true
+	 * @param callable $usernameGetter Function that takes the username of a sysop user and returns the username to
+	 * use for testing.
+	 * @param Status $validity Result of the password validity check
+	 * @param StatusValue $expect1 Expected result with $checkData = false
+	 * @param StatusValue $expect2 Expected result with $checkData = true
 	 */
-	public function testProviderAllowsAuthenticationDataChange( $type, $user, \Status $validity,
-		\StatusValue $expect1, \StatusValue $expect2
+	public function testProviderAllowsAuthenticationDataChange( $type, callable $usernameGetter, Status $validity,
+		StatusValue $expect1,
+		StatusValue $expect2
 	) {
+		$user = $usernameGetter( $this->getTestSysop()->getUserIdentity()->getName() );
 		if ( $type === PasswordAuthenticationRequest::class ) {
 			$req = new $type();
 			$req->password = 'NewPassword';
@@ -414,43 +432,78 @@ class LocalPasswordPrimaryAuthenticationProviderTest extends \MediaWikiIntegrati
 			$provider->providerAllowsAuthenticationDataChange( $req, false )
 		);
 		$this->assertEquals(
-			$expect2->getValue() === 'ignored' ? $expect2 : \StatusValue::newFatal( 'badretype' ),
+			$expect2->getValue() === 'ignored' ? $expect2 : StatusValue::newFatal( 'badretype' ),
 			$provider->providerAllowsAuthenticationDataChange( $req, true )
 		);
 
 		$provider = $this->getProvider( true );
 		$this->assertEquals(
-			\StatusValue::newGood( 'ignored' ),
+			StatusValue::newGood( 'ignored' ),
 			$provider->providerAllowsAuthenticationDataChange( $req, true ),
 			'loginOnly mode should claim to ignore all changes'
 		);
 	}
 
 	public static function provideProviderAllowsAuthenticationDataChange() {
-		$err = \StatusValue::newGood();
+		$err = StatusValue::newGood();
 		$err->error( 'arbitrary-warning' );
 
 		return [
-			[ AuthenticationRequest::class, 'UTSysop', \Status::newGood(),
-				\StatusValue::newGood( 'ignored' ), \StatusValue::newGood( 'ignored' ) ],
-			[ PasswordAuthenticationRequest::class, 'UTSysop', \Status::newGood(),
-				\StatusValue::newGood(), \StatusValue::newGood() ],
-			[ PasswordAuthenticationRequest::class, 'uTSysop', \Status::newGood(),
-				\StatusValue::newGood(), \StatusValue::newGood() ],
-			[ PasswordAuthenticationRequest::class, 'UTSysop', \Status::wrap( $err ),
-				\StatusValue::newGood(), $err ],
-			[ PasswordAuthenticationRequest::class, 'UTSysop', \Status::newFatal( 'arbitrary-error' ),
-				\StatusValue::newGood(), \StatusValue::newFatal( 'arbitrary-error' ) ],
-			[ PasswordAuthenticationRequest::class, 'DoesNotExist', \Status::newGood(),
-				\StatusValue::newGood(), \StatusValue::newGood( 'ignored' ) ],
-			[ PasswordDomainAuthenticationRequest::class, 'UTSysop', \Status::newGood(),
-				\StatusValue::newGood( 'ignored' ), \StatusValue::newGood( 'ignored' ) ],
+			[
+				AuthenticationRequest::class,
+				static fn ( $sysopUsername ) => $sysopUsername,
+				Status::newGood(),
+				StatusValue::newGood( 'ignored' ),
+				StatusValue::newGood( 'ignored' )
+			],
+			[
+				PasswordAuthenticationRequest::class,
+				static fn ( $sysopUsername ) => $sysopUsername,
+				Status::newGood(),
+				StatusValue::newGood(),
+				StatusValue::newGood()
+			],
+			[
+				PasswordAuthenticationRequest::class,
+				'lcfirst',
+				Status::newGood(),
+				StatusValue::newGood(),
+				StatusValue::newGood()
+			],
+			[
+				PasswordAuthenticationRequest::class,
+				static fn ( $sysopUsername ) => $sysopUsername,
+				Status::wrap( $err ),
+				StatusValue::newGood(),
+				$err
+			],
+			[
+				PasswordAuthenticationRequest::class,
+				static fn ( $sysopUsername ) => $sysopUsername,
+				Status::newFatal( 'arbitrary-error' ),
+				StatusValue::newGood(),
+				StatusValue::newFatal( 'arbitrary-error' )
+			],
+			[
+				PasswordAuthenticationRequest::class,
+				static fn () => 'DoesNotExist',
+				Status::newGood(),
+				StatusValue::newGood(),
+				StatusValue::newGood( 'ignored' )
+			],
+			[
+				PasswordDomainAuthenticationRequest::class,
+				static fn ( $sysopUsername ) => $sysopUsername,
+				Status::newGood(),
+				StatusValue::newGood( 'ignored' ),
+				StatusValue::newGood( 'ignored' )
+			],
 		];
 	}
 
 	/**
 	 * @dataProvider provideProviderChangeAuthenticationData
-	 * @param callable|bool $usernameTransform
+	 * @param callable|false $usernameTransform
 	 * @param string $type
 	 * @param bool $loginOnly
 	 * @param bool $changed
@@ -466,8 +519,12 @@ class LocalPasswordPrimaryAuthenticationProviderTest extends \MediaWikiIntegrati
 		$oldpass = $testUser->getPassword();
 		$newpass = 'NewPassword';
 
-		$dbw = wfGetDB( DB_PRIMARY );
-		$oldExpiry = $dbw->selectField( 'user', 'user_password_expires', [ 'user_name' => $cuser ] );
+		$dbw = $this->getDb();
+		$oldExpiry = $dbw->newSelectQueryBuilder()
+			->select( 'user_password_expires' )
+			->from( 'user' )
+			->where( [ 'user_name' => $cuser ] )
+			->fetchField();
 
 		$this->mergeMwGlobalArrayValue( 'wgHooks', [
 			'ResetPasswordExpiration' => [ static function ( $user, &$expires ) {
@@ -557,7 +614,11 @@ class LocalPasswordPrimaryAuthenticationProviderTest extends \MediaWikiIntegrati
 			$expectExpiry,
 			wfTimestampOrNull(
 				TS_MW,
-				$dbw->selectField( 'user', 'user_password_expires', [ 'user_name' => $cuser ] )
+				$dbw->newSelectQueryBuilder()
+					->select( 'user_password_expires' )
+					->from( 'user' )
+					->where( [ 'user_name' => $cuser ] )
+					->fetchField()
 			)
 		);
 	}
@@ -574,7 +635,7 @@ class LocalPasswordPrimaryAuthenticationProviderTest extends \MediaWikiIntegrati
 	}
 
 	public function testTestForAccountCreation() {
-		$user = \User::newFromName( 'foo' );
+		$user = User::newFromName( 'foo' );
 		$req = new PasswordAuthenticationRequest();
 		$req->action = AuthManager::ACTION_CREATE;
 		$req->username = 'Foo';
@@ -584,27 +645,27 @@ class LocalPasswordPrimaryAuthenticationProviderTest extends \MediaWikiIntegrati
 
 		$provider = $this->getProvider();
 		$this->assertEquals(
-			\StatusValue::newGood(),
+			StatusValue::newGood(),
 			$provider->testForAccountCreation( $user, $user, [] ),
 			'No password request'
 		);
 
 		$this->assertEquals(
-			\StatusValue::newGood(),
+			StatusValue::newGood(),
 			$provider->testForAccountCreation( $user, $user, $reqs ),
 			'Password request, validated'
 		);
 
 		$req->retype = 'Baz';
 		$this->assertEquals(
-			\StatusValue::newFatal( 'badretype' ),
+			StatusValue::newFatal( 'badretype' ),
 			$provider->testForAccountCreation( $user, $user, $reqs ),
 			'Password request, bad retype'
 		);
 		$req->retype = 'Bar';
 
 		$this->validity->error( 'arbitrary warning' );
-		$expect = \StatusValue::newGood();
+		$expect = StatusValue::newGood();
 		$expect->error( 'arbitrary warning' );
 		$this->assertEquals(
 			$expect,
@@ -615,14 +676,14 @@ class LocalPasswordPrimaryAuthenticationProviderTest extends \MediaWikiIntegrati
 		$provider = $this->getProvider( true );
 		$this->validity->error( 'arbitrary warning' );
 		$this->assertEquals(
-			\StatusValue::newGood(),
+			StatusValue::newGood(),
 			$provider->testForAccountCreation( $user, $user, $reqs ),
 			'Password request, not validated, loginOnly'
 		);
 	}
 
 	public function testAccountCreation() {
-		$user = \User::newFromName( 'Foo' );
+		$user = User::newFromName( 'Foo' );
 
 		$req = new PasswordAuthenticationRequest();
 		$req->action = AuthManager::ACTION_CREATE;
@@ -632,7 +693,7 @@ class LocalPasswordPrimaryAuthenticationProviderTest extends \MediaWikiIntegrati
 		try {
 			$provider->beginPrimaryAccountCreation( $user, $user, [] );
 			$this->fail( 'Expected exception was not thrown' );
-		} catch ( \BadMethodCallException $ex ) {
+		} catch ( BadMethodCallException $ex ) {
 			$this->assertSame(
 				'Shouldn\'t call this when accountCreationType() is NONE', $ex->getMessage()
 			);
@@ -641,7 +702,7 @@ class LocalPasswordPrimaryAuthenticationProviderTest extends \MediaWikiIntegrati
 		try {
 			$provider->finishAccountCreation( $user, $user, AuthenticationResponse::newPass() );
 			$this->fail( 'Expected exception was not thrown' );
-		} catch ( \BadMethodCallException $ex ) {
+		} catch ( BadMethodCallException $ex ) {
 			$this->assertSame(
 				'Shouldn\'t call this when accountCreationType() is NONE', $ex->getMessage()
 			);
@@ -676,14 +737,10 @@ class LocalPasswordPrimaryAuthenticationProviderTest extends \MediaWikiIntegrati
 		$expect->createRequest->username = 'Foo';
 		$this->assertEquals( $expect, $provider->beginPrimaryAccountCreation( $user, $user, $reqs ) );
 
-		// We have to cheat a bit to avoid having to add a new user to
-		// the database to test the actual setting of the password works right
-		$dbw = wfGetDB( DB_PRIMARY );
-
-		$user = \User::newFromName( 'UTSysop' );
+		$user = $this->getTestSysop()->getUser();
 		$req->username = $user->getName();
 		$req->password = 'NewPassword';
-		$expect = AuthenticationResponse::newPass( 'UTSysop' );
+		$expect = AuthenticationResponse::newPass( $user->getName() );
 		$expect->createRequest = $req;
 
 		$res2 = $provider->beginPrimaryAccountCreation( $user, $user, $reqs );
